/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * * 
 * Use is subject to license terms.
 ******************************************************************/
package isac.gui;

import isac.bridge.CalcTree;
import isac.gui.treetable.CalcModelHierarchy;
import isac.gui.treetable.CalcModelNode;
import isac.gui.treetable.CalculationModel;
import isac.gui.treetable.CustomTreeCellRenderer;
import isac.interfaces.ICalcIterator;
import isac.interfaces.IGuiToGui;
import isac.interfaces.IToGUI;
import isac.interfaces.IToUser;
import isac.useractions.IUIAction;
import isac.useractions.IUIElement;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UIActionOnCalcElement;
import isac.useractions.UIActionOnIterator;
import isac.useractions.UserAction;
import isac.useractions.UserActionOnCalcElement;
import isac.useractions.UserActionOnIterator;
import isac.util.CalcChangedEvent;
import isac.util.Formalization;
import isac.util.formulae.CalcHead;
import isac.util.formulae.KEStoreID;
import isac.util.formulae.CalcFormula;
import isac.util.tactics.Tactic;
import isac.wsdialog.IWorksheetDialog;
import isac.wsdialog.DialogProtocolException;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.plaf.basic.BasicInternalFrameUI;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeSelectionModel;

import org.apache.log4j.Logger;

/**
 * <code>Worheet</code> is the component that is responsible for the
 * interaction with the user while doing a calculation. Furthermore it acts as a
 * visual represantiation of mathematical formulas in a calculation tree, that
 * is beeing executed by the isac knowledge interpreter.
 * 
 * The <code>Worksheet</code> is the view of the <code>WorksheetDialog</code>.
 * 
 * The data, that the <code>WorksheetDialog</code> delivers, is representated
 * in the hierarchical structur <code>JTree</code> or on the
 * <code>CalcHeadPanel</code>.
 * 
 * @author Mario Hochreiter 2003
 * @author Alois Kirchsteiger 2004
 * @author Manuel Koschuch 2005
 * @version 0.1 15/09/2003
 */
public class Worksheet extends JPanel implements ActionListener,
		CellEditorListener, Serializable, InternalFrameListener, IGuiToGui {

	/**
	 * for serialization purposes
	 */
	private static final long serialVersionUID = 1L;

	private MouseAdapter ml;

	private JMenuItem applicableTacticsMenuItem_;

	private JMenuItem tactic_applied_menuitem_;

	private JMenuItem assumptionsMenuItem_;

	private JMenuItem intermediate_steps_menuitem_;

	private JPopupMenu popupMenu;

	private JPopupMenu applicableTacticsMenu_;

	private Point mouse_point_;

	private Component mouse_component_;

	private Vector applicableTactics_;

	private IToWorksheetUser windowApplication_;

	// Vector containing the actions that should be applied on all calcElements
	private Vector dynamic_actions_on_calcElements_;

	// ****HashMaps for UIActions on CalcElments
	// HashMap containing the actions with the menu_items as key
	private HashMap dynamic_actions_by_menu_items_;

	// ****HashMap for UIActions on OneElement
	// contains actions with the iterator as key
	private HashMap dynamic_actions_by_iterator_;

	// *************Contanier for dynamic buttons -- CONTEXT_CALCHEAD
	private Vector dynamic_action_ids_with_context_calchead_;

	// private HashMap dynamic_actions_context_calchead_by_menu_items_;

	static Logger logger = Logger.getLogger(Worksheet.class.getName());

	protected IToUser worksheet_rmi_1_;

	protected IToGUI worksheet_rmi_2_;

	/**
	 * variable that says if the calculation has ended just to inhibit <Next>
	 * <Auto>
	 */
	protected boolean calculationEnded;

	// formula which will be edited if the dialog allows it
	protected CalcFormula edited_formula_;

	/** Model for the JTree */
	protected TreeModel model;

	/**
	 * LK+WN0505 remove: only used for getting DialogIterators; WorksheetDialog
	 * knows CalcTree, get Iterator from there
	 */
	protected CalcTree calc_tree_;

	protected CalcModelHierarchy calc_;

	/** Used to represent the model. */
	protected JTree tree;

	/** Dialog which is responsible for the communication with the kernel */
	protected IWorksheetDialog dialog_;

	/**
	 * true if it is allowed to call try_match_/try_refine_, false otherwiese
	 * WN0504 should be due to UserSettings
	 * 
	 * WN050421 kept due to isCallProblemAllowed used in initMouseListener
	 * WN+LK0504: made superfluous: remove ?
	 */
	protected boolean call_problem_allowed_;

	/**
	 * true if in single stepping mode WN0504 should be due to UserSettings
	 * WN+LK0504: made superfluous: remove
	 */
	protected boolean single_stepping_;

	protected JPanel button_panel_;

	/** Panel which is shows the calc_head_ if forced */
	protected CalcHeadPanel chp_;

	/** Panle which shows the tactic if forced */
	protected TacticPanel tp;

	/** submit calc_head_ and start calculation */
	protected JButton start_solving_;

	/** check calc head */
	protected JButton checkCalcHead;

	/** complete calc head */
	protected JButton compCalcHead;

	protected JButton show_calchead_;

	/**
	 * the position in the CalcTree any changes (by User + mathEngine) are
	 * applied to
	 */
	protected ICalcIterator active_formula_;

	protected CalcHead ch_;

	// LK 04.11.18
	protected int current_row_;

	protected Locale l_;

	private JDesktopPane desktop_;

	private boolean calc_head_visible_;

	private JInternalFrame panel_frame_;

	private int x_;

	private int y_;

	private ICalcIterator selected_node_;

	private static final int CHP_Y_POSITION_OFFSET = 17;

	/**
	 * Create a <code>Worksheet</code> (and display the Specification if
	 * requested by the viewStyle according to UserSettings). The given
	 * <code>DGuide</code> is connected to the <code>Worksheet</code>.
	 * 
	 * @param l
	 *            Locale that is used.
	 * @param dialogGuide
	 *            The dialog guide that is connected to the Worksheet
	 * @param f
	 *            Formalization necessary to start a new calculation
	 * @param source
	 *            Where was the calculation started...
	 * @param desktop
	 *            DesktopPane, the Worksheet is added to
	 * @see ...{isac.wsdialog.WorksheetDialog#STARTFROM_*}
	 */
	public Worksheet(IToWorksheetUser windowApplication, final Locale l,
			IWorksheetDialog dialogGuide, Formalization f, int source,
			JDesktopPane desktop) {
		// LK 04.11.18 (1 line)
		desktop_ = desktop;
		calc_head_visible_ = false;
		windowApplication_ = windowApplication;
		l_ = l;
		// initialize the Containers for UIActions
		dynamic_actions_by_iterator_ = new HashMap();
		dynamic_actions_by_menu_items_ = new HashMap();
		dynamic_actions_on_calcElements_ = new Vector();
		dynamic_action_ids_with_context_calchead_ = new Vector();
		current_row_ = 0;
		super.setLayout(new BorderLayout());
		single_stepping_ = true;// WN050420 remove
		call_problem_allowed_ = true;// FIXXXME.WN050420 remove
		dialog_ = dialogGuide;
		try {
			worksheet_rmi_1_ = new WorksheetRMI(this);
			worksheet_rmi_2_ = new WorksheetRMI(this);
			dialog_.addDataChangeListener(worksheet_rmi_1_);
			dialog_.registerUIControlListener(worksheet_rmi_2_);

			calc_tree_ = dialog_.getCalcTree(0, f, source);
			// WN050421-AK: Worksheet should _ONLY_ know about DialogIterator
			// and use notifyUserAction for Calculation-related sutff.
			// But THIS system-generated call still requires calc_tree_ !!
			// WN050727 redesign startCalculation such that a CalcChangedEvent
			// is thrown (thus same handling for root- and sub-pbl!)
			this.replaceCalcHeadPanelByWorksheet();
			dialog_.startCalculate();
			dialog_.notifyGUIReady();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	
	protected String getWorksheetID(){
	    String ws_id = null;
	    try {
	        ws_id =  dialog_.getWorksheetID();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return ws_id;
	}
	
	/**
	 * buildMenu to be called by clicking right mousebutton
	 */
	protected void buildPopupMenu(ICalcIterator it) {
		popupMenu = new JPopupMenu();

		// applicableTacticsMenuItem_ = new JMenuItem("applicable tactics");
		// applicableTacticsMenuItem_.addActionListener(this);
		// tactic_applied_menuitem_ = new JMenuItem("tactic applied");
		// tactic_applied_menuitem_.addActionListener(this);
		// assumptionsMenuItem_ = new JMenuItem("assumptions");
		// assumptionsMenuItem_.addActionListener(this);
		// intermediate_steps_menuitem_ = new JMenuItem("intermediate steps");
		// intermediate_steps_menuitem_.addActionListener(this);
		// popupMenu.add(applicableTacticsMenuItem_);
		// popupMenu.add(tactic_applied_menuitem_);
		// popupMenu.add(assumptionsMenuItem_);
		// popupMenu.add(intermediate_steps_menuitem_);

		// get the popup menus that are equivalet for all calc elements
		Iterator iterator = dynamic_actions_on_calcElements_.iterator();
		while (iterator.hasNext()) {
			IUIAction action = (IUIAction) iterator.next();
			String label = action.getLabel();
			JMenuItem menu_item = new JMenuItem(label);
			dynamic_actions_by_menu_items_.put(menu_item, action);
			menu_item.addActionListener(this);
			popupMenu.add(menu_item);
		}
		try {
			// get the popup menu items spezific for this CalcElement
			IUIAction action = (IUIAction) dynamic_actions_by_iterator_.get(it
					.getPosition().toSMLString());
			// WN050729 ..toSMLString left from following a wrong path
			// searching an error FIXXXME remove toSMLString
			if (action != null) {
				String label = action.getLabel();
				JMenuItem menu_item = new JMenuItem(label);
				dynamic_actions_by_menu_items_.put(menu_item, action);
				menu_item.addActionListener(this);
				popupMenu.add(menu_item);
			}
			// init mouse listener
		} catch (RemoteException ex) {
			ex.printStackTrace();
		}
		this.initMouseListener();
	}

	protected void buildApplicableTacticsMenu() {
		applicableTacticsMenu_ = new JPopupMenu();
		Iterator it = applicableTactics_.iterator();
		Integer step = new Integer(0);
		while (it.hasNext()) {
			Tactic tactic = (Tactic) it.next();
			JMenuItem item = new JMenuItem(tactic.toSMLString());
			item.setActionCommand(step.toString());
			item.addActionListener(this);
			applicableTacticsMenu_.add(item);
			int stepInt = step.intValue();
			stepInt++;
			step = new Integer(stepInt);
		}
		applicableTacticsMenu_.show(mouse_component_, mouse_point_.x,
				mouse_point_.y);
	}

	protected void initMouseListener() {
		ml = new MouseAdapter() {

			public void mousePressed(MouseEvent e) {
				logger.info("mouseEvent");

				// ensure, that worksheet is alwas in the background, when
				// CalcHeadPanel is visible
				if (calc_head_visible_) {
					moveToBack();
				} else {
					if (e.getButton() == MouseEvent.BUTTON3
							&& (tree.getSelectionPath() != null)) {

						x_ = e.getX();
						y_ = e.getY();
						CalcModelNode node = (CalcModelNode) tree
								.getSelectionPath().getLastPathComponent();
						ICalcIterator it = node.getValue();
						buildPopupMenu(it);
						selected_node_ = it;
						mouse_point_ = e.getPoint();
						mouse_component_ = e.getComponent();
						popupMenu.show(e.getComponent(), x_, y_);
					}
				}
			}
		};
		tree.addMouseListener(ml);
	}

	// move Worksheet to back and CalcHeadPanel to front
	private void moveToBack() {
		desktop_.moveToBack(this);
		this.setFocusable(false);
		desktop_.moveToFront(panel_frame_);
	}

	private void showTacticPanel() {
		if (tp == null) {
			tp = new TacticPanel();
			super.add(tp, BorderLayout.EAST);
		}
	}

	/* LK 04.09.02 */
	public boolean isCalculationEnded() {
		return this.calculationEnded;
	}

	/**
	 * TreeView; attachs the Worksheet to the CalcHeadPanel
	 */
	public void setTreeTableModel() {
		calc_ = new CalcModelHierarchy();
		model = new CalculationModel(calc_, calc_tree_.iterator());
		tree = new JTree(model);
		tree.setEditable(true);
		tree.setCellRenderer(new CustomTreeCellRenderer());
		tree.setCellEditor(new DefaultTreeCellEditor(tree,
				new DefaultTreeCellRenderer()));
		// LK 04.11.17: (1 line)
		tree.getCellEditor().addCellEditorListener(this);
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setRootVisible(false);
		tree.setShowsRootHandles(true);
		KeyStroke ks = KeyStroke.getKeyStroke(KeyEvent.VK_A,
				KeyEvent.CTRL_DOWN_MASK);
		tree.getInputMap().put(ks, "none");
		super.add(new JScrollPane(tree), BorderLayout.CENTER);
	}

	/**
	 * Do matching (see UC 2.5.6).
	 * 
	 * @param problemID
	 *            unique id for a CalcHead
	 */
	public void tryMatch(KEStoreID problemID) {
		// try {
		// // dialog.try_match_(ch, problemID);
		// dialog_.notifyUserAction(new UserActionOnKEStoreID(
		// IUIElement.UI_SPECIFY_TRY_MATCH, problemID));
		// logger.debug("CalcHead after try_match_: " + ch_.toSMLString());
		// chp_.fillTextFieldsFromCalcHead(ch_);
		// this.showCalcHeadPane();
		// } catch (RemoteException e) {
		// e.printStackTrace();
		// // } catch (NotInSpecificationPhaseException e) {
		// // e.printStackTrace();
		// } catch (DialogProtocolException e) {
		// logger.error(e.getMessage(), e);
		// }
	}

	/**
	 * Refine a problem (see UC 2.5.7).
	 * 
	 * @param problemID
	 *            unique id for a CalcHead
	 */
	// method at the moment only gets CalcHead but does not
	// mark the refined problem in the problem hierarchy_
	public void tryRefine(KEStoreID problemID) {
		// try {
		// // dialog.try_refine_(ch, problemID);
		// dialog_.notifyUserAction(new UserActionOnKEStoreID(
		// IUIElement.UI_SPECIFY_TRY_REFINE, problemID));
		// chp_.fillTextFieldsFromCalcHead(ch_);
		// this.showCalcHeadPane();
		// } catch (DialogProtocolException e) {
		// logger.error(e);
		// } catch (RemoteException e) {
		// logger.error(e);
		// }
	}

	/**
	 * Show all applicable Tactics that can be used for the next calculation
	 * step (UC 2.5.11).
	 * 
	 * @param scope
	 *            The filter parameter selects the scope of search for
	 *            applicable tactics.
	 */
	public void showApplicableTactics(int scope) {
		// FIXME.WN0504 use a DialogIterator !!!
		// try {
		// Tactic[] tactic = dialog_.fetchApplicableTactics(scope);
		// //TODO: show tactis in a list the user can select
		// } catch (RemoteException e) {
		// logger_.error(e);
		// }
	}

	/**
	 * @return true if the calculation has started.
	 */
	public boolean isCalculationStarted() {
		return active_formula_ != null;
	}

	private void setIterator() {
		try {
			active_formula_ = calc_tree_.iterator();
			// FIXXME.WN05011 active_formula_ = dialog_.iterator(); !!!
			active_formula_.moveRoot();
		} catch (RemoteException e) {
			e.printStackTrace();
			System.exit(5);
		}
	}

	// /* LK 04.09.01 */
	// // to be called from the windowApplication
	// public void nextStep() {
	// try {
	// dialog_.notifyUserAction(new UserAction(IUIElement.UI_CALCULATE_1));
	// // singleStepping = true;
	// // Tactic propTactic;
	// // try {
	// // if (it_ == null)
	// // this.setIterator();
	// // propTactic = dialog.fetchProposedTactic();
	// // //FIXME:WN: vvvv ...endCalculation
	// // if (propTactic != null) {
	// // dialog.setNextTactic(propTactic);
	// // } else {
	// // this.calculationEnded = true;
	// // }
	// // } catch (RemoteException e1) {
	// // e1.printStackTrace();
	// // } //try
	// } catch (RemoteException e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// } catch (DialogProtocolException e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// }
	// }

	// /* LK 04.09.01 */
	// // to be called from the windowApplication
	// public void autoCalc() {
	// //if (!this.calculationEnded) {
	// try {
	// single_stepping_ = false;
	// dialog_.notifyUserAction(new UserAction(
	// IUserAction.UI_CALCULATE_ALL));
	// for (int i = 0; i < tree.getRowCount(); i++) {
	// tree.expandRow(i);
	// }
	// this.calculationEnded = true;
	// } catch (RemoteException e1) {
	// logger.error("actionPerformed", e1.detail);
	// } catch (DialogProtocolException e1) {
	// logger.error("actionPerformed");
	// } //try
	// //}//if
	// }

	/**
	 * replace the CalcHeadPanel by the Worksheet FIXXXME.WN050727 streichen, +
	 * das zugeh"orige setzen streichen!
	 */
	private void replaceCalcHeadPanelByWorksheet() {
		this.setIterator();
		this.setTreeTableModel();
		this.buildPopupMenu(active_formula_);
		this.showTacticPanel();// WN050421???
		this.validate();// WN050421???

	}

	/**
	 * 
	 * 
	 * Description of the method in HTML format, if a link can be given in the
	 * running text do this with {@link
	 * fully.qualified.Classname#methodName(fully.qualified.Paramclass)}
	 * 
	 * @param <paramname>
	 *            <paramdescription>
	 * @return <return value>
	 * @exception <exception>
	 *                <description when it is thrown>
	 * @see <fully.qualified.Classname#methodName(param-classes)>
	 */
	public void startSolving() {
		try {
			// dialog.newCalculation(ch);
			System.out.println("in startSolving()");
			chp_.fillCalcHeadFromTextFields(ch_);
			logger.fatal("WS->DG: UI_SPECIFY_TO_SOLVE");
			if (dialog_.notifyUserAction(new UserAction(
					IUserAction.UI_SPECIFY_TO_SOLVE))) {
				replaceCalcHeadPanelByWorksheet();
			} else {
				JOptionPane.showMessageDialog(this,
						"Please enter a correct CalcHead",
						"CalcHead not correct", JOptionPane.WARNING_MESSAGE);
			}
		} catch (RemoteException e1) {
			logger.error("actionPerformed", e1.detail);
		} catch (DialogProtocolException e2) {
			logger.error("actionPerformed", e2);
			System.out.println("e2" + e2);
		}
	}

	public void compCalcHead() {
		try {
			chp_.fillCalcHeadFromTextFields(ch_);
			dialog_.notifyUserAction(new UserAction(
					IUserAction.UI_SPECIFY_COMPLETE_CALCHEAD));
			chp_.fillTextFieldsFromCalcHead(ch_);
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	public void checkCalcHead() {
		try {
			logger.debug(dialog_);
			dialog_.notifyUserAction(new UserAction(
					IUserAction.UI_SPECIFY_CHECK_CALCHEAD));
			chp_.fillTextFieldsFromCalcHead(ch_);
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	/**
	 * shows the applicable tactics, if applied on the activeFormula a
	 * applicable tactic can be chosen and is applied to this formula
	 */
	public void applicableTactics() {
		try {
			CalcModelNode node = (CalcModelNode) tree.getSelectionPath()
					.getLastPathComponent();
			ICalcIterator it = node.getValue();
			applicableTactics_ = it.getApplicableTactics(0);
			buildApplicableTacticsMenu();
			// WN0504 DialogIterator checks wrt. Usersettings etc., thus NOT...
			// dialog.notifyUserAction(new UserAction(
			// IUserAction.UI_SOLVE_GET_APPLICABLE_TACTICS));
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	/**
	 * shows the Tactic that was applied to this formula
	 * 
	 */
	public void tacticApplied() {
		try {
			CalcModelNode node = (CalcModelNode) tree.getSelectionPath()
					.getLastPathComponent();
			node.tacticString = ((ICalcIterator) node.getValue()).getTactic()
					.getName();
			tree.repaint();
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	public void assumptions() {
		try {
			System.out.println("assumtions");
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	// /**
	// * WN050511->LK? explanation please
	// */
	// public void intermediateSteps() {
	// try {
	// CalcModelNode node = (CalcModelNode) tree.getSelectionPath()
	// .getLastPathComponent();
	// ICalcIterator it = node.getValue();
	// calc_tree_.intermediateSteps(it);
	// } catch (Exception e1) {
	// logger.error(e1);
	// }
	// }

	/**
	 * Adds a Button to the CalcHeadPanel
	 * 
	 * @param action
	 *            the action associated with the button
	 */
	private void addCalcHeadButton(IUIAction action) {
		JButton dynamic_button = new JButton(action.getLabel());
		dynamic_button.addActionListener(this);
		dynamic_actions_by_menu_items_.put(dynamic_button, action);

		chp_.getButtonPanel().add(dynamic_button);
		super.validate();
	}

	/**
	 * Shows CalcHeadPanel. Creates a new InternalFrame on the desktopPane the
	 * worksheet is added to and ensures, that this frames stays in front of the
	 * Worksheet as long as it is opened
	 * 
	 * @param calc_head
	 *            CalcHead to display in Panel
	 * 
	 */
	private void openCalcHeadPanel(CalcHead calc_head) {

		panel_frame_ = new JInternalFrame("CalcHeadPanel");
		panel_frame_.setClosable(false);
		panel_frame_.setResizable(true);
		panel_frame_.setVisible(true);
		// ((BasicInternalFrameUI) panel_frame_.getUI()).setNorthPane(null);

		chp_ = new CalcHeadPanel(l_, calc_head);

		panel_frame_.getContentPane().add(chp_, BorderLayout.CENTER);
		panel_frame_.validate();
		panel_frame_.setSize(400, 420);
		panel_frame_.addInternalFrameListener(this);

		desktop_.add(panel_frame_);
		desktop_.moveToFront(panel_frame_);
		panel_frame_.setLocation(x_, y_ + CHP_Y_POSITION_OFFSET);

		calc_head_visible_ = true;
		
		chp_.getModelPanel().addFocusLostHandler(this);

	}

	// ------------- Implementation of ActionListener ----------------
	public void actionPerformed(ActionEvent e) {
		UIAction action = (UIAction) dynamic_actions_by_menu_items_.get(e
				.getSource());

		int context = action.getContext();

		try {
			switch (context) {
			// if CalcHeadPanel Button is pressed: alwas send calchead created
			// by reading all Fields in the CalcHeadPanel
			case IUIAction.UI_CONTEXT_CALCHEAD:
				CalcHead calc_head = new CalcHead();
				chp_.fillCalcHeadFromTextFields(calc_head);
				dialog_.notifyUserAction(new UserActionOnCalcElement(action
						.getAction(), calc_head));
				break;
			case IUIAction.UI_CONTEXT_ONEELEMENT:
				dialog_.notifyUserAction(new UserActionOnCalcElement(action
						.getAction(), selected_node_.getElement()));
				break;
			default:
				dialog_.notifyUserAction(new UserAction(action.getAction()));
				break;
			}

		} catch (RemoteException ex1) {
			ex1.printStackTrace();
		} catch (DialogProtocolException ex2) {
			// TODO Auto-generated catch block
			ex2.printStackTrace();
		}

		// (e.getSource() == start_solving_) {
		// startSolving();
		// } else if (e.getSource() == show_calchead_) {
		// showCalcHeadPanel();
		//
		// } else if (e.getSource() == compCalcHead) {
		// compCalcHead();
		// } else if (e.getSource() == checkCalcHead) {
		// checkCalcHead();
		// } else if (e.getSource() == applicableTacticsMenuItem_) {
		// applicableTactics();
		// } else if (e.getSource() == tactic_applied_menuitem_) {
		// tacticApplied();
		// } else if (e.getSource() == assumptionsMenuItem_) {
		// assumptions();
		// } else if (e.getSource() == intermediate_steps_menuitem_) {
		// intermediateSteps();
		// } else if (e.getActionCommand() != null
		// && !e.getActionCommand().equals("")) {
		// int pos = Integer.parseInt(e.getActionCommand());
		// Tactic tactic = (Tactic) applicableTactics_.get(pos);
		// try {
		// dialog_.notifyUserAction(new UserActionOnCalcElement(
		// IUserAction.UI_SOLVE_SET_NEXT_TACTIC, tactic));
		// } catch (RemoteException e1) {
		// // TODO Auto-generated catch block
		// e1.printStackTrace();
		// } catch (DialogProtocolException e1) {
		// // TODO Auto-generated catch block
		// e1.printStackTrace();
		// }
		// } else {
		// //unexpected event source
		// logger.error("unexpected event source " + e.getSource());
		// }
	}

	public void calcChanged(CalcChangedEvent event) {
		logger.debug("single Stepping: " + single_stepping_);
		logger.debug(model);
		logger.debug("calc" + calc_);
		logger.debug(model);
		((CalculationModel) model).calcChanged(event);
		for (int i = 0; i < tree.getRowCount(); i++) {// WN050428 ???+DG
			// filters
			tree.expandRow(i);
		}
		tree.setEditable(true);
		try {
			dialog_.notifyGUIReady();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * if the user has canceled the editing, the worksheet should inform the
	 * dialogGuide
	 * 
	 * @see javax.swing.event.CellEditorListener#editingCanceled(javax.swing.event.ChangeEvent)
	 */
	public void editingCanceled(ChangeEvent e) {
		// TODO Auto-generated method stub

	}

	/**
	 * if the user has changed the value of a node, it should be altered
	 * 
	 * @see javax.swing.event.CellEditorListener#editingStopped(javax.swing.event.ChangeEvent)
	 */
	public void editingStopped(ChangeEvent e) {
		Object o = tree.getCellEditor().getCellEditorValue();
		try {
			CalcModelNode current_node = (CalcModelNode) tree
					.getSelectionPath().getLastPathComponent();
			active_formula_ = current_node.getValue();
			if (active_formula_ == null) { // append formula
				// appends after the currently active formula
				edited_formula_ = new CalcFormula();
				edited_formula_.setText((String) o);
				logger.fatal(" WS->DG: UI_SOLVE_APPEND_USER_FORMULA");
				dialog_.notifyUserAction(new UserAction(
						IUserAction.UI_SOLVE_APPEND_USER_FORMULA));
			} else { // replace formula
				logger.fatal(" WS->DG: UI_SOLVE_MOVE_ACTIVE_FORMULA");
				dialog_.notifyUserAction(new UserActionOnIterator(
						IUserAction.UI_SOLVE_MOVE_ACTIVE_FORMULA,
						active_formula_));
				edited_formula_ = new CalcFormula();
				edited_formula_.setText((String) o);
				edited_formula_.setPosition(active_formula_.getPosition());
				logger.fatal(" WS->DG: UI_SOLVE_EDIT_ACTIVE_FORMULA");
				dialog_.notifyUserAction(new UserAction(
						IUserAction.UI_SOLVE_EDIT_ACTIVE_FORMULA));
			}
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		System.out.println(o);
	}

	/**
	 * @param action
	 * @deprecated in favour of doUIAction(IUIAction action) with introduction
	 *             of IToGUI
	 */
	public boolean doUIAction(IUserAction action) {
		int actionID = action.getActionID();
		switch (actionID) {
		case IUIElement.UI_DO_EDIT_FORMULA:
			try {
				logger.fatal(" WS->DG: UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE");
				dialog_.notifyUserAction(new UserActionOnCalcElement(
						IUserAction.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
						edited_formula_));
			} catch (RemoteException e) {
				e.printStackTrace();
			} catch (DialogProtocolException e) {
				e.printStackTrace();
			}
			return true;
		case IUIElement.UI_DO_APPEND_FORMULA:
			try {
				logger.fatal(" WS->DG: UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE");
				dialog_.notifyUserAction(new UserActionOnCalcElement(
						IUserAction.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
						edited_formula_));
			} catch (RemoteException e) {
				e.printStackTrace();
			} catch (DialogProtocolException e) {
				e.printStackTrace();
			}
		default:
			return false;
		}

	}

	/**
	 * add an interactive element on the gui depending on the context
	 * 
	 * @see isac.interfaces.IToGUI#addUIElement(isac.wsdialog.IUIAction)
	 */
	public void addUIElement(IUIAction action) {
		int context = action.getContext();
		try {
			switch (context) {
			// TOODO: build all UIElements from wsdialog with this method
			case IUIAction.UI_CONTEXT_CALCULATION:
				windowApplication_.addToMenuBar(action);
				break;
			case IUIAction.UI_CONTEXT_CALCELEMENTS:
				dynamic_actions_on_calcElements_.add(action);
				break;
			case IUIAction.UI_CONTEXT_ONEELEMENT:
				// WN050729 ..toSMLString left from following a wrong path
				// searching an error FIXXXME remove toSMLString
				dynamic_actions_by_iterator_.put(((UIActionOnIterator) action)
						.getIterator().getPosition().toSMLString(), action);
				break;
			case IUIAction.UI_CONTEXT_CALCHEAD:
				if (!dynamic_action_ids_with_context_calchead_
						.contains(new Integer(action.getAction()))) {
					addCalcHeadButton(action);
				}
				break;
			}
		} catch (RemoteException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * remove an interactive element from the gui
	 * 
	 * @see isac.interfaces.IToGUI#removeUIElement(isac.wsdialog.IUIAction)
	 */
	public boolean removeUIElement(IUIAction action) {
		int context = action.getContext();
		switch (context) {
		// TOODO: build all UIElements from wsdialog with this method

		case IUIAction.UI_CONTEXT_CALCULATION:
			return windowApplication_.removeFromMenuBar(action);
		case IUIAction.UI_CONTEXT_CALCELEMENTS:
			break;
		case IUIAction.UI_CONTEXT_ONEELEMENT:
			break;
		case IUIAction.UI_CONTEXT_CALCHEAD:
			return windowApplication_.removeFromMenuBar(action);
		}
		return false;
	}

	/**
	 * notify UserActions which come from outside the Worksheet, i.e.
	 * WindowApplication#actionPerformed
	 */
	public void notifyUserAction(IUserAction action) {
		 //switch (action.getContext())
		// case IUIAction.UI_CONTEXT_CALCULATION:
		try {
			// dialog_.notifyUserAction(new UserAction(action.getAction()));
			dialog_.notifyUserAction(action);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (DialogProtocolException e) {
			e.printStackTrace();
		}
		// //all other contexts are irrelevant here
		// case IUIAction.UI_CONTEXT_CALCELEMENTS:
		// case IUIAction.UI_CONTEXT_ONEELEMENT:
		// case IUIAction.UI_CONTEXT_SWITCH_PHASES:
		// }
	}

	/**
	 * perform the UserAction requested from the WorksheetDialog
	 * 
	 * @see isac.interfaces.IToGUI#doUIAction(isac.wsdialog.IUIAction)
	 */
	public void doUIAction(IUIAction action) {
		System.out.println("in do UIAction, Context: " + action.getContext());
		switch (action.getContext()) {
		// TOODO: complete groups of UIElements
		case IUIAction.UI_CONTEXT_CALCULATION:
			break;
		case IUIAction.UI_CONTEXT_CALCELEMENTS:
			break;
		case IUIAction.UI_CONTEXT_ONEELEMENT:
			break;
		case IUIAction.UI_CONTEXT_SWITCH_PHASES:
			if (!calc_head_visible_) {
				CalcHead calc_head = (CalcHead) (((UIActionOnCalcElement) action)
						.getCalcElement());
				openCalcHeadPanel(calc_head);

			} else {
				closeCalcHeadPanel();
			}
			break;
		case IUIAction.UI_CONTEXT_CALCHEAD:
			chp_
					.fillTextFieldsFromCalcHead((CalcHead) (((UIActionOnCalcElement) action)
							.getCalcElement()));
			break;
		}
	}

	private void closeCalcHeadPanel() {
		System.out.println("in close");
		panel_frame_.dispose();
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
	}

	public void internalFrameClosed(InternalFrameEvent e) {
		System.out.println("closed");
		calc_head_visible_ = false;
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameActivated(InternalFrameEvent e) {
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
	}


}