/*
 * Created on 18.07.2005
 *
 */
package isac.gui.browser;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.HashMap;

import isac.browserdialog.BDialog;
import isac.gui.browser.panels.TabPanel;
import isac.interfaces.IGuiToGui;
import isac.interfaces.IToGUI;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UserAction;
import isac.wsdialog.DialogProtocolException;

import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JMenuBar;

/**
 * A (in the WindowApplication) displayable browser frame.
 * Holds a TabPanel and a BrowserWindow and a set of dynamic
 * generated buttons.
 * 
 * @author JL
 * @version $Revision$
 *
 */
public class BrowserFrame extends JInternalFrame implements IToGUI, ActionListener, Serializable, IGuiToGui {

	private BrowserWindow browser_window_;
	
	private TabPanel browser_panel_;

    private HashMap dynamic_buttons_;
    private HashMap dynamic_actions_;
    
    /**
     * each browser frame has to know his dialog
     */
    protected BDialog browser_dialog_;
	
	/**
	 * Constructs the BrwoserFrame with the right TabPanel (eg. .for the
	 * problem browser)
	 * 
	 * @param title The displayed title of the window
	 * @param browser_panel the contetnt panel
	 **/
	public BrowserFrame(String title, TabPanel browser_panel) {
		super(title, false, true, false, false);
		browser_panel_ = browser_panel;
		browser_window_ = new BrowserWindow(browser_panel);
		this.getContentPane().add(browser_window_);
		this.setJMenuBar(new JMenuBar());
		browser_panel_.setBrowserInterface(this);
		
        dynamic_actions_ = new HashMap();
        dynamic_buttons_ = new HashMap();
        
	}
	
	/**
	 * To add dynamic buttons to the menu of the frame.
	 * 
	 * @param action The action assigned to the button.
	 */
    private void addToMenuBar(IUIAction action) {
        int action_id = action.getAction();
        //check if a button is already in the menu
        if (! dynamic_buttons_.containsKey(new Integer(action_id)) ) {
            JButton dynamic_button = new JButton(action.getLabel());      
            dynamic_button.setName(action.getLabel());
            dynamic_button.addActionListener(this);
            
            dynamic_buttons_.put(new Integer(action_id), dynamic_button);
            dynamic_actions_.put(dynamic_button, action);
            getJMenuBar().add(dynamic_button);
            getJMenuBar().updateUI();
        }
    }

	/** 
	 * Handles all events of the dynamic buttons
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 * 
	 * @TOOODO JL: implement actions
	 **/
	public void actionPerformed(ActionEvent action_event) {
		System.out.println("BrowserFrame::ayctionPerformend()");
		UIAction action = (UIAction)dynamic_actions_.get(action_event.getSource());
			try {
				browser_dialog_.notifyUserAction(new UserAction(action.getAction()));
			} catch (RemoteException remote_exception) {
				// TODO Auto-generated catch block
				remote_exception.printStackTrace();
			} catch (DialogProtocolException dialog_protocol_exception) {
				// TODO Auto-generated catch block
				dialog_protocol_exception.printStackTrace();
			}
	}

	/**
	 * Removes item from menu. 
	 * 
	 * @param action
	 * @return true if done else false
	 */
	private boolean removeFromMenuBar(IUIAction action) {
		if(!dynamic_buttons_.containsKey( new Integer(action.getAction()))){
			return false;
		} else {
			JButton button_do_delete = (JButton) dynamic_buttons_.get(new Integer(action.getAction()));
			getJMenuBar().remove(button_do_delete);
            getJMenuBar().updateUI();
			dynamic_buttons_.remove(new Integer(action.getAction()));
			dynamic_actions_.remove(button_do_delete);
			return true;
		}
	}

	/* (non-Javadoc)
	 * @see isac.interfaces.IToGUI#activateUIObject(int, boolean)
	 */
	public void activateUIObject(int id, boolean active) throws RemoteException {
		// TODO Auto-generated method stub
		
	}

	/**
	 * @deprecated
	 * @see isac.interfaces.IToGUI#doUIAction(isac.useractions.IUserAction)
	 */
	public boolean doUIAction(IUserAction action) throws RemoteException {
		// TODO Auto-generated method stub
		return false;
	}

	/* 
	 * @see isac.interfaces.IToGUI#doUIAction(isac.useractions.IUIAction)
	 */
	public void doUIAction(IUIAction action) throws RemoteException {
		System.out.println("BrowserFrame::doUIAction() context="+ action.getContext());
		switch (action.getContext()) {
			case (IUIAction.UI_CONTEXT_PROBLEM_BROWSER_PANEL):
				browser_panel_.doUIAction(action);
				break;
			case (IUIAction.UI_CONTEXT_PROBLEM_BROWSER_MINIBROWSER):
				// do something with the mini browser
				break;
		}
		
	}

	/** 
	 * to add a user interface element
	 * 
	 * @see isac.interfaces.IToGUI#addUIElement(isac.useractions.IUIAction)
	 */
	public void addUIElement(IUIAction action) throws RemoteException {
		if(action.getContext() == IUIAction.UI_CONTEXT_PROBLEM_BROWSER) {
			addToMenuBar(action);
		}
		
	}

	/** 
	 * removes a user interface element
	 * 
	 * @see isac.interfaces.IToGUI#removeUIElement(isac.useractions.IUIAction)
	 */
	public boolean removeUIElement(IUIAction action) throws RemoteException {
		if(action.getContext() == IUIAction.UI_CONTEXT_PROBLEM_BROWSER) {
			boolean result = removeFromMenuBar(action);
			return result;
		}
		return false;
	}

	/**
	 * Each browser frame has to know his dialog (for handling of the
	 * dynamic actions and buttons). Therefore the dialog has to be
	 * registered to the browser frame
	 * 
	 * @param browser_dialog dialog to register
	 */
	public void registerBrowserDialog(BDialog browser_dialog) {
		browser_dialog_ = browser_dialog;
	}

	/**
	 * To tell the BrowserFrame that some of its members has done something.
	 * It can decide what to do but it has to know something about the action.
	 * Note that the action can be a action of a none dynamic generated button!
	 * 
	* @see isac.interfaces.IGuiToGui#notifyUserAction(isac.useractions.IUserAction)
	 */
	public void notifyUserAction(IUserAction action) {
		System.out.println("BrowserFrame::notifieUIAction() "+ action.getActionID());
		
		try {
			browser_dialog_.notifyUserAction(action);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DialogProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	

}
