/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.browser.panels;

import isac.gui.WindowApplication;
import isac.gui.util.Hierarchy;
import isac.gui.util.ProblemNode;
import isac.useractions.IUIAction;
import isac.useractions.IUIElement;
import isac.useractions.IUserAction;
import isac.useractions.UserActionOnKEStoreID;
import isac.useractions.UserActionOnKEStoreIDAndWSDID;
import isac.useractions.UserActionOnString;
import isac.util.formulae.KEStoreID;

import java.awt.event.*;
import java.rmi.RemoteException;

import javax.swing.*;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;

/**
 * @author Mario Hochreiter, JL
 * @version 0.1
 *  
 */
public class ProblemBrowserPanel extends TabPanel {

	static Logger logger = Logger
			.getLogger(ProblemBrowserPanel.class.getName());

	protected ProblemNode sel_node_;

	protected TreePath sel_path_;

	public ProblemBrowserPanel(JFrame root, Hierarchy hierarchy) {
		super(root, hierarchy);
		this.initMouseListener();
	}


	/**
	 * initializes the mouse listener and calls the necessary functions
	 */
	protected void initMouseListener() {
		ml_ = new MouseAdapter() {

			public void mousePressed(MouseEvent e) {
				int selRow = view_tree_.getRowForLocation(e.getX(), e.getY());
				if (selRow != -1 && selRow != 0) {
					sel_path_ = view_tree_.getPathForLocation(e.getX(), e.getY());
					sel_node_ = (ProblemNode) sel_path_.getLastPathComponent();
					if ((e.getClickCount() == 1)) {
						setNewBrowserContent();
					} else if (e.getClickCount() == 2) {
						// do nothing
					} else if (e.getButton() == MouseEvent.BUTTON3) {
						// do nothing
					}
				}
			}
		};
		view_tree_.addMouseListener(ml_);
	}
	
	/**
	 * To set the new content of the browser. Private function generates the
	 * necessary UserAction to get a new content from the dialog.
	 */
	private void setNewBrowserContent() {
		UserActionOnKEStoreIDAndWSDID action = new UserActionOnKEStoreIDAndWSDID(
				IUIAction.UI_PROBLEMBROWSER_SELECTION_CHANGED,
				getSelectedKeStoreId(), getTopWSDId());
		ui_interface_.notifyUserAction(action);
	}

	
	/**
	 * returns the id of the WorkSheetDialog, that handles the Worksheet on the
	 * top of the WindowApplication
	 * 
	 * @return
	 */
	private String getTopWSDId() {
		return ((WindowApplication)getTopLevelAncestor()).getActiveWSDID();
	}

	/**
	 * returns the KEStoreID of the selected problem
	 * 
	 * @return id of the selected problem
	 */
	private KEStoreID getSelectedKeStoreId() {
		KEStoreID ke_store_id = new KEStoreID();
		Object[] path = sel_path_.getPath();
		for (int i = path.length - 1; i > 0; i--) {
			ke_store_id.addString(((ProblemNode) path[i]).getId());
		}
		return ke_store_id;
	}


	/* (non-Javadoc)
	 * @see isac.interfaces.IToGUI#activateUIObject(int, boolean)
	 */
	public void activateUIObject(int id, boolean active) throws RemoteException {
		// TODO Auto-generated method stub
		
	}


	/**
	 * @deprecated
	 * @see isac.interfaces.IToGUI#doUIAction(isac.useractions.IUserAction)
	 */
	public boolean doUIAction(IUserAction action) throws RemoteException {
		// TODO Auto-generated method stub
		return false;
	}


	/**
	 * @see isac.interfaces.IToGUI#doUIAction(isac.useractions.IUIAction)
	 */
	public void doUIAction(IUIAction action) throws RemoteException {
		System.out.println("ProblemBrowserPanel::doUIAction() action="+ action.getAction());
		switch(action.getAction()){
			case(IUIElement.UI_PROBLEMBROWSER_SELECTED_ID): {
				sendSelectedId();
				break;
			}
			case(IUIElement.UI_PROBLEMBROWSER_ACTIVE_WORKSHEET): {
				sendActiveWorksheetId();
				break;
			}
		}
	}


	/**
	 * Sends the active worksheet id to the dialog
	 * 
	 * @TOOODO send the real id
	 */
	private void sendActiveWorksheetId() {
		System.out.println("ProblemBrowserPanel::sendActiveWorksheetId()");
		UserActionOnString action = new UserActionOnString(
				IUIElement.UI_PROBLEMBROWSER_ACTIVE_WORKSHEET,
				"TOOODO: send the WorksheetID");
		ui_interface_.notifyUserAction(action);
	}


	/**
	 * sends the ID of the selecte problem
	 */
	private void sendSelectedId() {
		System.out.println("ProblemBrowserpanel::sendSelectedId()");
		UserActionOnKEStoreID action = new UserActionOnKEStoreID(
				IUIElement.UI_PROBLEMBROWSER_SELECTED_ID,
				getSelectedKeStoreId());
		
		ui_interface_.notifyUserAction(action);
		
	}


	/* (non-Javadoc)
	 * @see isac.interfaces.IToGUI#addUIElement(isac.useractions.IUIAction)
	 */
	public void addUIElement(IUIAction action) throws RemoteException {
		// TODO Auto-generated method stub
		
	}


	/* (non-Javadoc)
	 * @see isac.interfaces.IToGUI#removeUIElement(isac.useractions.IUIAction)
	 */
	public boolean removeUIElement(IUIAction action) throws RemoteException {
		// TODO Auto-generated method stub
		return false;
	}
}