/**
 * @author Andreas Griesmayer
 * Created on Oct 6, 2003
 */
package isac.session;

import isac.browserdialog.BDialog;
import isac.wsdialog.IWorksheetDialog;

import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;

import org.apache.log4j.Logger;

/**
 * @author Andreas Griesmayer
 * @deprecated because Dinopolis did not become available
 */
public class ObjectManager extends UnicastRemoteObject implements
        IObjectManager {

    static Logger logger_ = Logger.getLogger(ObjectManager.class.getName());

    SessionManager sdialog_ = SessionManager.getInstance();

    /**
     * instantiates the Objectmanager which is used to simulate a function of
     * dinopolis The ObjectManager provides functions to fetch objects which are
     * used by other modules of the system using RMI
     * 
     * @throws RemoteException
     */
    protected ObjectManager() throws RemoteException {
        super();
        rmiBind();
    }

    /**
     * fetch the BrowserDialog identified by <code>session</code>
     */
    public BDialog getBrowserDialog(String session) throws RemoteException {
        return sdialog_.getBDialog(session);
    }

    /**
     * fetch the WorksheetDialog identified by <code>wdialog_id</code>
     */
    public IWorksheetDialog getWorksheetDialog(String wdialog_id)
            throws RemoteException {
        return (IWorksheetDialog) sdialog_.getWorksheetDialog(wdialog_id);
    }

    /**
     * fetch the SessionManager of the system - multiple calls to this method
     * result in one and the same SessionManager.
     */
    public ISessionManager getSessionDialog() throws RemoteException {
        if (sdialog_ == null)
            sdialog_ = SessionManager.getInstance();
        return sdialog_;
    }

    /*
     * instantiates the ObjectManager and registers it on the RMIRegistry on the
     * local machine as //localhost/isac-ObjectManager . If no RMIRegistry is
     * running, a new one is started.
     */
    public static void main(String[] args) {
        try {
            new ObjectManager();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    private void rmiBind() {
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        String name = "//localhost/isac-ObjectManager";
        try {
            System.out.println("try to bind as " + name);
            Naming.rebind(name, this);
            System.out.println("Object Manager bound to " + name);
        } catch (java.rmi.ConnectException e) {
            System.err.println("failed to contact as " + name
                    + " (creating RMI-Server on localhost:1099)");
            try {
                LocateRegistry.createRegistry(1099);
                rmiBind();
            } catch (java.rmi.RemoteException exc2) {
                System.err.println("can not create registry:"
                        + exc2.getMessage());
            }
        } catch (Exception exc) {
            System.err.println("failed to connect to rmiregistry");
            System.err.println(exc);
        }
    }

}